function [firecolor,clrpro]= FireColorMask(frameRGB,clrmodel)
% Flame color model function
% input : 
% frameRGB : an RGB color image
% clrmodel : a struct 
% clrmodel.name : specifiying the color model
%       'HSI': H < th(default pi/3), I > Imean;
%       'cb/cr' : Cb/Cr < th(default 0.5);
%       'YCbCr' : Y > Cb, Cr > Cb, Y > Ymean, Cb < Cbmean, Cr > Crmean,|Cb - Cr|<40;
%       'RGB' : R > G >= B, R > R_T, S >= ((255-R)*S_T/R_T); 
%       'FixedGMM' : within a distance of two sigma to any component
%       'DPGMMRGB' : regression based on a GMM trained by a DP, returning probabilities
% clrmodel.th : threshold of color models
% Written by Zhenglin Li 
%%       
     [hgt,wdh] = size(frameRGB(:,:,1));
     firecolor = zeros(hgt,wdh);
     clrpro = zeros(hgt,wdh);
     pnum = wdh * hgt;    

%% HSI color model
if strcmp(clrmodel.name,'HSI')
     if ~isfield(clrmodel,'th'), clrmodel.th = pi/3; end % Setting default threshold
     frameHSI = rgb2hsi(frameRGB);
     Imean = mean(mean(frameHSI(:,:,3)));
     firecolor(frameHSI(:,:,1)<= clrmodel.th) = 1;
     firecolor(frameHSI(:,:,1) <= 0) = 0; 
     firecolor( frameHSI(:,:,3) == 0) = 0;
     firecolor(frameHSI(:,:,3) < Imean) = 0;
     
elseif strcmp(clrmodel.name,'cb/cr')
  %%  cb/cr color model
      if ~isfield(clrmodel,'th'), clrmodel.th = 0.5; end % Setting default threshold
      frameycbcr = rgb2ycbcr(frameRGB);      
      firecolor(frameycbcr(:,:,2)./frameycbcr(:,:,3) < clrmodel.th) = 1;

elseif strcmp(clrmodel.name, 'YCbCr')
%%   Ycbcr color model
    if ~isfield(clrmodel,'th'), clrmodel.th = 40; end  % Setting default threshold   
    frameycbcr = rgb2ycbcr(frameRGB); 
    Y = frameycbcr(:,:,1);
    Cb = frameycbcr(:,:,2);
    Cr = frameycbcr(:,:,3);
    tmp = abs(Cr - Cb);
    
%     clrpro = double(tmp)/255;
    firecolor(Y > Cb) = 1;
    firecolor(Cr <= Cb) = 0;
%     clrpro(Y <= Cb) = 0;
%     clrpro(Cr <= Cb) = 0;
    
    Ymean = sum(sum(Y))/pnum;
    Cbmean = sum(sum(Cb))/pnum;
    Crmean = sum(sum(Cr))/pnum;
    
    firecolor(Y <= Ymean) = 0;
    firecolor(Cb >= Cbmean) = 0;
    firecolor(Cr <= Crmean) = 0;
    
%     clrpro(Y <= Ymean) = 0;
%     clrpro(Cb >= Cbmean) = 0;
%     clrpro(Cr <= Crmean) = 0;
    
    firecolor( tmp < clrmodel.th) = 0;
    
    
%     fu = [-2.6e-10
%           3.3e-7
%           -1.7e-4
%           5.16e-2
%           -9.1
%           9.6e2
%           -5.6e4
%           1.4e6];
%     Crpro = zeros(hgt,wdh,8);
%     Crpro(:,:,8)= ones(hgt,wdh);
%     Crpro(:,:,7)= Cr;
%     tmp = double(Cr);
%     for i = 1:6
%         tmp = double(Cr).*tmp;
%         Crpro(:,:,7-i) = tmp;
%     end
%     fu = permute(fu,[3 2 1]);
%     fu_cr = bsxfun(@times,fu,Crpro);
%     fu_cr = sum(fu_cr,3);
%     fu_cr1 = zeros(hgt,wdh);
%     for i = 1:hgt
%         for j = 1:wdh
%             fu_cr1(i,j) = -2.6e-10 * Cr(i,j)^7+ 3.3e-7 * Cr(i,j)^6-1.7e-4 *...
%                 Cr(i,j)^5+5.16e-2 * Cr(i,j)^4-9.1* Cr(i,j)^3+9.6e2* Cr(i,j)^2-5.6e4 * Cr(i,j)+1.4e6;
%         end
%     end
%     firecolor(Cb < fu_cr1)=0;
elseif strcmp(clrmodel.name,'RGB')
%%     RGB color model
    if ~isfield(clrmodel,'thr'), clrmodel.thr = 125; end % Setting default threshold
    if ~isfield(clrmodel,'ths'),  clrmodel.ths = 60; end 
    R = frameRGB(:,:,1);
    G = frameRGB(:,:,2);
    B = frameRGB(:,:,3);
    frameHSV = rgb2hsv(frameRGB);
    S = frameHSV(:,:,2);
    firecolor(R >= G) = 1;
    firecolor(G <= B) = 0;
    firecolor(R <= clrmodel.thr) = 0;
    firecolor(100*S < ((255*ones(hgt,wdh)- 255*im2double(R))*clrmodel.ths/clrmodel.thr)) = 0;

elseif strcmp(clrmodel.name,'FixedGMM')
%% GMM with preset component number
    mu = [121.16 76.87 43.98
          169.08 84.14 35.20
          177.00 104.00 62.00
          230.42 113.78 43.71
          254.47 214.66 83.08
          254.97 159.06 151.00
          254.98 140.98 141.93
          254.99 146.95 102.99
          255.00 174.08 175.01
          255.00 217.96 176.07];

      Var = [101.16
             102.77
             100.00
             107.22
             100.11
             100.08
             100.39
             99.57
             101.01
             100.78];
      for i = 1:hgt
        for j = 1:wdh  
         pxltmp = double(permute(frameRGB(i,j,:),[1 3 2]));
         pxltmp = repmat(pxltmp,10,1);
         dis = sum((pxltmp-mu).*(pxltmp-mu), 2) - 12*Var;
         firecolor(i,j) = 1-isempty(find((dis<0),1));
        end
      end
      
elseif strcmp(clrmodel.name,'DPGMMRGB')
%% DPGMM in the RGB colour space
    if ~isfield(clrmodel,'th'), clrmodel.th = -14.441; end % Setting default threshold
    % Clr_thres = 0.8; % 0.8 is the threshold in the paper
    % clr_ampl = 1.5e6;   
    for j = 1:hgt
        rowtmp = double(permute(frameRGB(j,:,:),[2 3 1]));
        rowprob = pdf(clrmodel.GMM,rowtmp);
        clrpro(j,:) = rowprob';
    end
    clrpro = log(clrpro);
    firecolor(clrpro<=(clrmodel.th)) = 0;
    firecolor(clrpro>(clrmodel.th)) = 1; 
    
elseif strcmp(clrmodel.name,'DPGMMYCbCr') 
    %% GMM in the YCbCr colour space
    frameycbcr = rgb2ycbcr(frameRGB); 
    frameycbcr = frameycbcr(:,:,2:3);

    clr_ampl = 1e4;
    clrpro = zeros(hgt,wdh);
    for j = 1:hgt
        rowtmp = double(permute(frameycbcr(j,:,:),[2 3 1]));
        rowprob = pdf(GMM,rowtmp);
        clrpro(j,:) = rowprob';
    end
    clrpro = clrpro*clr_ampl;
    firecolor(clrpro<=Clr_thres) = 0;
    firecolor(clrpro>Clr_thres) = 1; 
end
